# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys
sys.path.insert(0, os.path.abspath('../synadm/../'))


# -- Project information -----------------------------------------------------

project = 'synadm'
copyright = '2020-2025, Johannes Tiefenbacher'
author = 'Johannes Tiefenbacher'

# The short X.Y version
version = '0.48'

# The full version, including alpha/beta/rc tags
release = '0.48'


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.napoleon',
    'sphinx_click',
    'myst_parser',
    # 'sphinx.ext.autosectionlabel',
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = 'en'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []


# -- Options for HTML output -------------------------------------------------
html_theme = "pydata_sphinx_theme"

html_context = {
    "codeberg_user": "synadm",
    "codeberg_repo": "synadm",
    "codeberg_version": "master",
    # XXX: may have issues with files in folders? not sure, we don't have any
    "edit_page_url_template": "https://codeberg.org/{{ codeberg_user }}/{{ codeberg_repo }}/_edit/{{ codeberg_version }}/{{ doc_path }}{{ file_name }}{{ some_other_arg }}",
    "edit_page_provider_name": "Codeberg",
    "doc_path": "doc/source/",
}
html_theme_options = {
    "icon_links": [
        {
            "name": "Codeberg",
            "url": "https://codeberg.org/synadm/synadm",
            "icon": "fa-solid fa-code-branch",
        },
    ],
    "use_edit_page_button": True,

    # this is a weird hack that makes site headers not overflow (and be
    # problematic), avoid changing without checking if the site header
    # became bigger!
    "navbar_align": "left",

    "header_links_before_dropdown": 3,
    "show_toc_level": 3,
    "back_to_top_button": False,
}
html_title = "synadm"
html_static_path = ["_static"]
html_css_files = ["synadm.css"]


# -- Extension configuration -------------------------------------------------
myst_heading_anchors = 3
